/*
 * Copyright 2001-2002, 2010-2011 pilot <pilot@monkey.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>

#include <ruby.h>
#include "arirang.h"
#include "version.h"

char *html = NULL;
FILE *htmlreport;
int wflags = 0;
int fflags = 0;
int ftype = 0;
int sslflags = 0;
short int CONNECT_TIMEOUT = 3;
short int RECV_TIMEOUT = 5;
int scan_count = 0;
int rule_grab = 0;
int rflags = 0;
int pflags = 0;
int ruby = 0;
int isnetwork = 0;
char *ariruby = NULL;
FILE *rubyfd;

int proxy = 0;
char *proxyhost = NULL;
int proxyport = 3128;

VALUE rb_cArirang;
VALUE rb_arirang;

TAILQ_HEAD(ports_list, scan_port) portsqueue;

int
main(int argc, char *argv[])
{
    int pcount = 0;

    unsigned short int MAX_PROCESSES = 60;
    unsigned short int RULE_PROCESSES = 0;
    unsigned short int IPRANGE_PROCESSES = 60;

    unsigned int http_port = 80;
    char *qport = NULL;
    struct scan_port *sport;

    int hostcount = 0;
    unsigned long start = 0, end = 0, count;
    unsigned short int statuscode = 1;

    int ch;
    char *server = NULL;
    char *filename = NULL;
    FILE *scanlist, *tmp_iprange;
    char list[50];

    char *rulefile = NULL;
    int gflags = 0, oflags = 0, hflags = 0,
        mflags = 0;

    struct hostent *host_cidr;
    struct in_addr host_addr;
    int cidr_domain = 0;

    char *slash = NULL;
    unsigned long cidr =0, mask=0;
    int bits = 0;

    unsigned long fcount = 0;
    char *fip;
    int fnum = 0;

    char *xport = NULL;
    char *env_type = NULL;
    char proxy_type[32];
   
    ID arirang_ID;
    ID ari_config_ID;
 
    unsigned long sec;
    unsigned long usec;
    struct timeval tv1, tv2;
    gettimeofday(&tv1, 0);

    if (argc < 3)
        usage();

    scrview();
    opterr = 0;
    TAILQ_INIT(&portsqueue);
    
    while ((ch = getopt(argc, argv, "T:X:St:w:R:GOp:f:h:s:e:r:vP:")) != -1) {
        switch (ch) {
        case 'X':
            proxy = 1;
            proxyhost = optarg;
            if ((xport = strchr(proxyhost, ':')) != NULL) {
                proxyport = atoi(xport+1);
                *xport = '\0';
            } 
            break;
        case 'R':
            ruby = 1;
            ariruby = optarg;
            break;
        case 'S':
            sslflags = 1;
            http_port = 443;
            break;
        case 't':
            RECV_TIMEOUT = atoi(optarg);
            break;
        case 'T':
            CONNECT_TIMEOUT = atoi(optarg);
            break;
        case 'w':
            wflags = 1; 
            html = optarg;
            break;
        case 'G':
            gflags = 1;
            break;
        case 'O':
            oflags = 1;
            break;
        case 'p':
            pflags = 1;
            qport = strtok(optarg, ",");
            while (qport != NULL) {
                sport = (struct scan_port *)malloc(sizeof(struct scan_port));
                sport->port = atoi(qport);
                TAILQ_INSERT_TAIL(&portsqueue, sport, port_next); 
                qport = strtok(NULL, ",");
            }   
            http_port = atoi(optarg);
#ifdef DEBUG
            TAILQ_FOREACH(sport, &portsqueue, port_next) {
                printf("portsqueue = %d \n", sport->port);
            }
#endif
            break;
        case 'f':
            fflags = 1;
            filename = optarg;
            break;
        case 'h':
            hflags = 1;
            server = optarg;
            break;
        case 's':
            start = inet_addr(optarg);
            break;
        case 'e':
            end = inet_addr(optarg);
            break;
        case 'r':
            rflags = 1;
            rulefile = optarg;
            break;
        case 'v':
            statuscode = 2;
            break;
        case 'P':
            MAX_PROCESSES = atoi(optarg);
            RULE_PROCESSES = atoi(optarg);
            IPRANGE_PROCESSES = atoi(optarg);
            break;
        default:
            usage();
        }
    }

#ifdef DEBUG
    printf("proxy = %d, proxyhost = %s, proxyport = %d\n", proxy, proxyhost, proxyport);
    printf("CONNECT_TIMEOUT: %d \n", CONNECT_TIMEOUT);
    printf("RECV_TIMEOUT: %d \n", RECV_TIMEOUT);
    printf("wflags: %d \n", wflags);
    printf("sslflags: %d \n", sslflags);
    printf("gflags: %d \n", gflags);
    printf("oflags: %d \n", oflags);
    printf("ports: %d \n", http_port);
    printf("fflags: %d \n", fflags);
    printf("ruby: %d \n", ruby);
    printf("ariruby: %s \n", ariruby);
#endif

/* default 80/tcp, ssl 443/tcp */
    if (pflags == 0) {
        sport = (struct scan_port *)malloc(sizeof(struct scan_port));

        if (sslflags)
            sport->port = 443;
        else
            sport->port = 80;
   
        TAILQ_INSERT_TAIL(&portsqueue, sport, port_next);
    }

    if (hflags) {
        if ((slash = strchr(server, '/')) != NULL)
            mflags = 1;
    }   

    if (start && end) {
        if (start > end) {
            fprintf(stderr, "start end address incorrect \n\n");
            exit(0);
        }
    }

    if (MAX_PROCESSES > 512 || RULE_PROCESSES > 512 || IPRANGE_PROCESSES > 512) {
        fprintf(stderr, "too many processes \n\n");
        exit(0);
    }
   
    if (sslflags && ruby == 0 ) 
        fprintf(stdout, "starting arirang %s + SSL    %s \n\n", VERSION, HOMEPAGE);
    else if (sslflags && ruby)
        fprintf(stdout, "starting arirang %s + SSL + Ruby    %s \n\n", VERSION, HOMEPAGE);
    else if (ruby)
        fprintf(stdout, "starting arirang %s + Ruby   %s \n\n", VERSION, HOMEPAGE);
    else
        fprintf(stdout, "starting arirang %s    %s \n\n", VERSION, HOMEPAGE);

    if (proxy) {
        if ((env_type = getenv("PROXY_TYPE")) != NULL) 
           snprintf(proxy_type, sizeof(proxy_type), "%s", env_type);

        if (strstr(proxy_type, "SOCKS5") != NULL)
            fprintf(stdout, "arirang using socks5 proxy - %s:%d\n\n", proxyhost, proxyport);
        else
            fprintf(stdout, "arirang using http proxy - %s:%d\n\n", proxyhost, proxyport);
    }

    if (rflags)
        fprintf(stdout, "%s - %s\n\n", SCANRULE_MSG, rulefile);

    if ((ruby && wflags) || (ruby && rflags)) {
        fprintf(stderr, "arirang script doesn't support with -w or -r \n");
        exit(1);
    }

    if (ruby) {
        if ((rubyfd = fopen(ariruby, "rt")) == NULL) {
            fprintf(stderr, "arirang script - %s read error \n", ariruby);
            exit(1);
        }
        fprintf(stdout, "arirang script file - %s \n", ariruby);

    /* load baedal */
    signal(SIGINT, connout_r);

    ariruby_init(ariruby);
    arirang_ID = rb_intern("Arirang");
    ari_config_ID = rb_intern("baedal");

    ariruby_run();
    rb_cArirang = rb_const_get(rb_cObject, arirang_ID);
    rb_arirang = rb_class_new_instance(0, NULL, rb_cArirang);

    rb_funcall(rb_arirang, ari_config_ID, 0);
    fprintf(stdout, "\n");

    }

    if (mflags) {
        *slash++ = '\0'; 

/* check domain or ip */
        fip = server;
        while (*fip != '\0') {
        if ((*fip >= 'a' && *fip <= 'z')  || (*fip >= 'A' && *fip <= 'Z'))
            fnum = 1;
         fip++;
        }
        if (fnum == 1) {
            if ((host_cidr = gethostbyname(server)) == NULL) {
                fprintf(stderr, "host not found \n\n");
                exit(0);
            }

        for (hostcount = 0; host_cidr->h_addr_list[hostcount] != NULL; hostcount++) {
#ifdef DEBUG
            printf ("host count = %d ", hostcount);
#endif
            host_addr.s_addr  = *((unsigned long* ) host_cidr->h_addr_list[hostcount]);
            if (hostcount == 0)
                fprintf(stdout, "%s - %s * \n", server, inet_ntoa(host_addr));
            else
                fprintf(stdout, "%s - %s  \n", server, inet_ntoa(host_addr));   
        }
/* first host list scan */
        host_addr.s_addr = *((unsigned long* ) host_cidr->h_addr_list[0]);
        server = inet_ntoa(host_addr);
        cidr_domain = 1;
        }

        cidr = inet_addr(server);

        if (fnum != 1 || cidr_domain) {
        bits = atoi(slash);

        if (bits > 0 && bits < 33) { 
            mask = 0xFFFFFFFF << (32 - bits);
#ifdef DEBUG
            printf("bits: %d,  mask: %lu\n", bits, mask);
#endif
            cidr = ntohl(cidr);
            start = htonl(cidr & mask);
            end = htonl(cidr | ~mask);
        }
        else {
            fprintf(stderr, "bits incorrect \n\n");
            exit(0);
        }
      }
    fnum = 0;
    }

    if (wflags) {
      if ((htmlreport = fopen(html, "w+")) == NULL) {
          fprintf(stderr, "File %s write error\n", html);
          exit(1);
      }
      fprintf(stdout, "arirang scan report file - %s \n\n", html);
      htmlhead(html);
    }

    if (gflags && hflags && mflags == 0) {
        scan_count++;
        TAILQ_FOREACH(sport, &portsqueue, port_next) {
            grabhead(server, sport->port, 0);
        }
    }

    if (ruby && hflags && mflags == 0) {
        scan_count++;
        TAILQ_FOREACH(sport, &portsqueue, port_next) {
            rubyscan(server, sport->port, 0);
        }
    }


    if (oflags && hflags && mflags == 0) {
        scan_count++;
        TAILQ_FOREACH(sport, &portsqueue, port_next) {
            grabhead(server, sport->port, 1);
        }
    }

    if (fflags == 1 && rflags == 0 && mflags == 0) {

        if ((scanlist = fopen(filename, "rt")) == NULL) {
            fprintf(stderr, "File %s open error\n", filename);
            exit(0);
        }

/* -f with process */
        pcount = 0;
        while (!feof(scanlist)) {
            fscanf(scanlist, "%s\n", list);

/* check domain or ip */
        fip = list;
        while (*fip != '\0') {
        if ((*fip >= 'a' && *fip <= 'z')  || (*fip >= 'A' && *fip <= 'Z'))
            fnum = 1;
         fip++;
        }

        if (fnum == 0) {
            fcount = inet_addr(list);
            fcount = ntohl(fcount);
                                                                                          
            if ((fcount & 0xff) == 0 || (fcount & 0xff) == 255) 
                continue;
        }
        fnum = 0;

            if (pcount > MAX_PROCESSES)
                wait(NULL);

            switch (fork()) {
            case 0:
#ifdef DEBUG
            printf("%s \n", list);
#endif

            if (gflags) {
                TAILQ_FOREACH(sport, &portsqueue, port_next) {
                    grabhead(list, sport->port, 0);
                }
            }
            if (oflags) {
                TAILQ_FOREACH(sport, &portsqueue, port_next) {
                    grabhead(list, sport->port, 1);
                }
            }
            if (ruby) {
                TAILQ_FOREACH(sport, &portsqueue, port_next) {
                    rubyscan(list, sport->port, 0);
                }
            }
                exit(0);
            case -1:
                perror("fork err");
                exit(-1);
            default:
                pcount++;
                break;
            }
        }
        fclose(scanlist);

        while (pcount--) {
            scan_count++;
            wait(NULL);
        }
    }

    if ((start && end && rflags == 0) || (mflags == 1 && rflags == 0)) {
        pcount = 0;

        for (count = ntohl(start); count <= ntohl(end); count++) {
            if ((count & 0xff) == 0 || (count & 0xff) == 255)
                continue;
            if (pcount > MAX_PROCESSES)
                wait(NULL);

            switch (fork()) {
            case 0:
                if (oflags) {
                    TAILQ_FOREACH(sport, &portsqueue, port_next) {
                        grabheadW(count, sport->port, 0, 1);
                    }
                }
                else if (ruby) {
                    isnetwork = 1;
                    TAILQ_FOREACH(sport, &portsqueue, port_next) {
                        rubyscanW(count, sport->port, 0, 0);
                    }
                }
                else {
                    TAILQ_FOREACH(sport, &portsqueue, port_next) {
                        grabheadW(count, sport->port, 0, 0);
                    }
                }
                
                exit(0);
            case -1:
                perror("fork");
                exit(-1);
            default:
                pcount++;
                break;
            }
        }
        while (pcount--) {
            scan_count++;   
            wait(NULL);
        }
    }
    if (rflags && statuscode && hflags && mflags == 0) {
        if (RULE_PROCESSES == 0)
            scanrule(server, http_port, rulefile, statuscode);
        if (RULE_PROCESSES != 0)
            scanruleP(server, http_port, rulefile, statuscode, RULE_PROCESSES);
    }
/* -f with rule */
    if (rflags && statuscode && fflags && mflags == 0) {
        ftype = 1;
        if ((tmp_iprange = fopen(USED_IP_RANGE, "w")) == NULL) {
            fprintf(stderr, "File %s write error", USED_IP_RANGE);
            exit(1);
        }
        fclose(tmp_iprange);
        fprintf(stdout, "please wait - arirang make %s file for network scan\n\n", USED_IP_RANGE);

        if ((scanlist = fopen(filename, "rt")) == NULL) {
            fprintf(stderr, "File %s open error\n", filename);
            exit(0);
        }

        pcount = 0;
        while (!feof(scanlist)) {
            fscanf(scanlist, "%s\n", list);

/* check domain or ip */
        fip = list;
        while (*fip != '\0') {
        if ((*fip >= 'a' && *fip <= 'z')  || (*fip >= 'A' && *fip <= 'Z'))
            fnum = 1;
        fip++;
        }

        if (fnum == 0) {
            fcount = inet_addr(list);
            fcount = ntohl(fcount);

            if ((fcount & 0xff) == 0 || (fcount & 0xff) == 255)
                continue;
         }
        fnum = 0;

            if (pcount > MAX_PROCESSES)
                wait(NULL);

            switch (fork()) {
            case 0:
        
                if (gflags)
                    grabhead(list, http_port, 0);
                if (oflags)
                    grabhead(list, http_port, 1);

                exit(0);
            case -1:
                perror("fork err");
                exit(-1);
            default:
                pcount++;
                break;
            }
        }
        fclose(scanlist);

        while (pcount--) {
            scan_count++;
            wait(NULL);
        }

        ftype = 0;

        if ((scanlist = fopen(USED_IP_RANGE, "rt")) == NULL) {
            fprintf(stderr, "File %s open error\n", USED_IP_RANGE);
            exit(1);
        }
        while (!feof(scanlist)) {
            fscanf(scanlist, "%s\n", list);
            rule_grab = 1;
            if (oflags)
                grabhead(list, http_port, 1);
            else
                grabhead(list, http_port, 0);

            rule_grab = 0;
            scanruleP(list, http_port, rulefile, statuscode, IPRANGE_PROCESSES); 
        }
        fclose(scanlist);
    }

    if ((rflags && start && end) || (rflags && mflags == 1)) {
        if ((tmp_iprange = fopen(USED_IP_RANGE, "w")) == NULL) {
            fprintf(stderr, "File %s write error", USED_IP_RANGE);
            exit(1);
        }
        fclose(tmp_iprange);
        fprintf(stdout, "please wait - arirang make %s file for network scan\n\n", USED_IP_RANGE);

        pcount = 0;

        for (count = ntohl(start); count <= ntohl(end); count++) {
            if ((count & 0xff) == 0 || (count & 0xff) == 255)
                continue;
            if (pcount > IPRANGE_PROCESSES)
                wait(NULL);

            switch (fork()) {
            case 0:
                if (oflags)
                    grabheadW(count, http_port, 1, 1);
                else
                    grabheadW(count, http_port, 1, 0);
                exit(0);
            case -1:
                perror("fork");
                exit(-1);
            default:
                pcount++;
                break;
            }
        }
        while (pcount--) {
            scan_count++;
            wait(NULL);
        }

        if ((scanlist = fopen(USED_IP_RANGE, "rt")) == NULL) {
            fprintf(stderr, "File %s open error\n", USED_IP_RANGE);
            exit(1);
        }
        while (!feof(scanlist)) {
            fscanf(scanlist, "%s\n", list);
            rule_grab = 1;
            if (oflags)
                grabhead(list, http_port, 1);
            else
                grabhead(list, http_port, 0);

            rule_grab = 0;
            scanruleP(list, http_port, rulefile, statuscode, IPRANGE_PROCESSES);
        }
        fclose(scanlist);

    }

    while ((sport = TAILQ_FIRST(&portsqueue)) != NULL) {
        TAILQ_REMOVE(&portsqueue, sport, port_next);
        free(sport);
    }
    
    gettimeofday(&tv2, 0);
    if (tv2.tv_usec <= tv1.tv_usec) { 
        tv2.tv_sec--;
        tv2.tv_usec = tv2.tv_usec + 1000000;
    } 
    sec = tv2.tv_sec - tv1.tv_sec;
    usec = (tv2.tv_usec - tv1.tv_usec) / 1000; 
    if (scan_count == 1)
        fprintf(stdout, "\narirang scan result: %d host %ld.%ld seconds\n\n", scan_count, sec, usec);
    else
        fprintf(stdout, "\narirang scan result: %d hosts %ld.%ld seconds\n\n", scan_count, sec, usec);

    return 0;
}
