/*
 * Copyright 2010 pilot <pilot@monkey.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>

#include "arirang.h"

extern int sock;
extern short int CONNECT_TIMEOUT;

void
sslconnout(int sig)
{
#ifdef DEBUG
    printf("ssl connout \n");
#endif
    exit(0);
}


SSL_CTX *
ssl_init_ctx(void)
{
    SSL_CTX *ctx;

    SSL_library_init();
    SSL_load_error_strings();
    SSLeay_add_ssl_algorithms();
    ctx = SSL_CTX_new(SSLv23_client_method());

    return ctx; 
}

SSL *
ssl_init_ssl(SSL_CTX *ctx, int sock)
{
    SSL *ssl;
    
    ssl = SSL_new(ctx);   
    SSL_set_fd(ssl, sock);


    signal(SIGALRM, sslconnout);
    alarm(CONNECT_TIMEOUT);
    if(SSL_connect(ssl) <= 0) {
#ifdef DEBUG
        fprintf(stderr, "ssl connect error\n"); 
#endif
        exit(0);
    }
    alarm(0);

    return ssl;
}

void
ssl_free(SSL_CTX *ctx, SSL *ssl)
{
    if (ssl) {
        SSL_shutdown(ssl);
        SSL_free(ssl);
        SSL_CTX_free(ctx);
    }
}

int 
ssl_write_timeout(int sock, SSL *ssl, char *buf, int len, int timeout)
{
/*
    read buffer SSL_write and use fds.
    arirang do not need ssl_write_timeout
*/
    return 1;
}

int
ssl_read_timeout(int sock, SSL *ssl, char *buf, int len, int timeout)
{
    fd_set  fds;
    int n, err;
    struct timeval  tv;
    
    FD_ZERO(&fds);
    FD_SET(sock, &fds);
    tv.tv_sec = timeout;
    tv.tv_usec = 0;

    n = select(sock + 1, &fds, NULL, NULL, &tv);
    if (n == 0)
        return -2;
    if (n == -1)
        return -1;

    err = SSL_read(ssl, buf, len);
    if (err > 0)
        return err;
    else
        return -1;
}

void
ssl_error(SSL *ssl, int ret, char *scanhost)
{
    int error = SSL_get_error(ssl, ret);

    switch (error) {
    case SSL_ERROR_NONE:
#ifdef DEBUG
        fprintf(stdout, "%s - ssl error none \n", scanhost);
#endif
        break;
    case SSL_ERROR_ZERO_RETURN:
        fprintf(stderr, "%s - ssl zero return \n", scanhost);
        exit(0); 
        break;
    default:
        fprintf(stderr, "%s - ssl error \n", scanhost);
        exit(0);
    }
}

