/*
 * Copyright 2001-2002, 2010 pilot <pilot@monkey.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/utsname.h>

#include <stdio.h>
#include <stdlib.h>

#include "arirang.h"
#include "version.h"

void
scrview(void)
{
    struct utsname  uts;
    uname(&uts);
    
    fprintf(stdout, "\033[0;32m%s \033[0m\033[0;35m  arirang %s for %s (%s)\033[0m \033[0m\n",
        TITLE, VERSION, uts.sysname, uts.machine);
    fprintf(stdout, "2001,2002,2010,2011 by pilot released %s  powered by twwwscan\n", RELEASE_DATE);
    fprintf(stdout, "                                                                                   \n"); 
}

void
usage(void)
{
    scrview();

    fprintf(stdout, "usage        : arirang [-Tt] [-w] [-X] [-S] [-RGO] [-p] [-hfs] [-e] [-r] [-v] [-P]  \n");
    fprintf(stdout, "<timeout>    : -T  socket connect timeout (default 3/sec)                           \n");
    fprintf(stdout, "             : -t  socket receive timeout (default 5/sec)                           \n");
    fprintf(stdout, "<html>       : -w  arirang scan result to html format                               \n");
    fprintf(stdout, "<proxy>      : -X  proxyserver[:port] (default port 3128/tcp)                       \n");
    fprintf(stdout, "             :     environment variables PROXY_USER, PROXY_PASS, PROXY_TYPE         \n");
    fprintf(stdout, "             :     default PROXY_TYPE is HTTP                                       \n");
    fprintf(stdout, "             :     cache   PROXY_TYPE is RELAY                                      \n");
    fprintf(stdout, "             :     socks5  PROXY_TYPE is SOCKS5                                     \n");
    fprintf(stdout, "<SSL>        : -S  use the SSL (default port 443/tcp)                               \n");
    fprintf(stdout, "<script>     : -R  arirang script (Ruby Language)                                   \n");
    fprintf(stdout, "<finger>     : -O  check the allow option of webserver                              \n");
    fprintf(stdout, "               -G  check server of webserver                                        \n");
    fprintf(stdout, "<port>       : -p  tcp port (default port 80/tcp) or multi-port 22,80,8080          \n");
    fprintf(stdout, "<scan type>  : -h  ip|host[/cidr] one host scan or CIDR scan                        \n");
    fprintf(stdout, "             : -f  file scan hosts list file                                        \n");
    fprintf(stdout, "             : -s  wide network scan start ip                                       \n");
    fprintf(stdout, "             : -e  wide network scan end   ip                                       \n");
    fprintf(stdout, "<scan rule>  : -r  arirang scan rule databases(.uxe)                                \n");
    fprintf(stdout, "<virtual>    : -v  virtual host scan and http 1.1 (rule scan)                       \n");
    fprintf(stdout, "<process>    : -P  max process count (file, network scan default 60 processes)      \n");
    fprintf(stdout, "                                                                                    \n");
    fprintf(stdout, "<examples>                                                                          \n");
    fprintf(stdout, "check server : arirang -G -h 127.0.0.1                                              \n");
    fprintf(stdout, "check allow  : arirang -O -h yourhost                                               \n");
    fprintf(stdout, "server port  : arirang -G -p 80,8080 -h yourhost                                    \n");
    fprintf(stdout, "script       : arirang -R script/server.rb -h yourhost/24                           \n");
    fprintf(stdout, "             : arirang -S -R script/allow.rb -h 192.168.0.1/16 -P 80                \n");
    fprintf(stdout, "             : arirang -R script/multiport.rb -p 22,80,8080 -h 192.168.0.1/23 -P 120\n");
    fprintf(stdout, "ssl scan     : arirang -S -G -h yourhost/30 -r scanrule/request.uxe                 \n");
    fprintf(stdout, "             : arirang -T 2 -S -G -s 192.168.1.1 -e 192.168.1.100                   \n");
    fprintf(stdout, "virtual      : arirang -G -h virtual_host -r test.uxe -v                            \n");
    fprintf(stdout, "scan hosts   : arirang -G -f scanlist_file                                          \n");
    fprintf(stdout, "             : arirang -R script/server.rb -f scanlist_file                         \n");
    fprintf(stdout, "proxy        : arirang -X proxyserver:1080 -S -G -h yourhost/30                     \n");
    fprintf(stdout, "             : arirang -X proxyserver:3128 -G -h yourhost/30 -r request.uxe         \n");
    fprintf(stdout, "             : arirang -X proxyserver -R script/iisver.rb -h yourhost/24            \n");
    fprintf(stdout, "network      : arirang -O -h 192.168.1.1/24                                         \n");
    fprintf(stdout, "               arirang -G -p 80,8080 -h yourdomain/24                               \n");
    fprintf(stdout, "               arirang -G -s 192.168.1.1 -e 192.168.2.255                           \n");
    fprintf(stdout, "scan rule    : arirang -O -h yourhost -r scanrule/unix.uxe                          \n");
    fprintf(stdout, "             : arirang -G -h yourhost -r scanrule/request.uxe                       \n");
    fprintf(stdout, "fast scan    : arirang -S -G -h yourhost -r nt.uxe -P 80                            \n");
    fprintf(stdout, "               arirang -G -f scanlist_file -r unicode.uxe -P 80                     \n");
    fprintf(stdout, "               arirang -T 2 -t 2 -G -s 192.168.1.1 -e 192.168.2.255 -r nt.uxe       \n");
    fprintf(stdout, "               arirang -G -s 192.168.1.1 -e 192.168.1.255 -r codered.uxe -P 80      \n");
    fprintf(stdout, "               arirang -S -G -h yourdomain/24 -r scanrule/request.uxe -P 80         \n");
    fprintf(stdout, "html report  : arirang -w result.html -S -O -h 192.168.1.1/16 -P 256                \n");
    fprintf(stdout, "               arirang -w result.html -G -s 192.168.1.1 -e 192.168.1.100 -P 100     \n");
    fprintf(stdout, "             : arirang -w report.html -G -h yourhost/24 -r scanrule/request.uxe -P 100  \n");
    fprintf(stdout, "                                                                                    \n");
    fprintf(stdout, "contact      : pilot@monkey.org    http://www.monkey.org/~pilot                     \n");
    fprintf(stdout, "                                                                                    \n");
    fprintf(stdout, "tested on %s", TESTED_OS);
    fprintf(stdout, "                                                                                    \n");
    fprintf(stdout, "                                                                                    \n");
    exit(1);
}
